local SA = AceLibrary("SpecialEvents-Aura-2.0")
local buffSalvation = GetSpellInfo(1038)
local buffGreaterSalvation = GetSpellInfo(25895)
local buffRighteousFury = GetSpellInfo(25780)

SalvationKiller = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")
local mod = SalvationKiller

local state = false
local PlayerClass

local function get(key)
	return mod.db.profile[key]
end

local function set(key, val)
	mod.db.profile[key] = val
end

local options
options = {
	type = "group",
	args = {
		action = {
			type = "text",
			name = "Action",
			desc = "What action should be taken when you receive Blessing of Salvation?",
			validate = {["query"] = "Show a prompt",
									["kill"] = "Always kill it",
									["keep"] = "Always keep it"},
			get = get,
			set = set,
			passValue = "Action",
		},
		whiletankonly = {
			type = "toggle",
			name = "Only when tanking",
			desc = "Only check when you are suppositly tanking",
			get = get,
			set = set,
			passValue = "WhileTankOnly",
		},
		recheck = {
			type = "toggle",
			name = "Reset when changing to tank",
			desc = "Rechecks if you go in to a tanking mode (Switch stance, gain Righteous Fury)",
			get = get,
			set = set,
			passValue = "Recheck",
		},
	},
}

function mod:OnEnable()
	self:RegisterDB("SalvationKillerDB")
	self:RegisterDefaults("profile", {
		Action = "query",
		Check = "raid",
		WhileTankOnly = true,
		Recheck = true,
	})

	self:RegisterChatCommand("/salvationkiller", "/salvkiller", "/boskiller", options)

	self:RegisterEvent("PLAYER_AURAS_CHANGED")
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
	self:RegisterEvent("SpecialEvents_PlayerBuffGained")


	StaticPopupDialogs["BosKiller"] = {
		text = "You are buffed with Blessing of Salvation!\nWhat do you want to do?",
		button1 = "Remove it",
		button2 = "Keep it",
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
		OnAccept = function() mod:CancelBuff() end
	}

	PlayerClass = select(2, UnitClass("player"))
end

function mod:CancelBuff()
	local name = GetPlayerBuffName(buffGreaterSalvation) or GetPlayerBuffName(buffSalvation)
	if name then
		CancelPlayerBuff(name)
		self:Print(("Removed %s!"):format(name))
	end
end

function mod:HasSalvation()
	local name = GetPlayerBuffName(buffGreaterSalvation) or GetPlayerBuffName(buffSalvation) or nil
	return name ~= nil
end

function mod:IsTanking()
	-- Bear form
	if PlayerClass == "DRUID" and GetShapeshiftForm(true) == 1 then
		return true
	end

	-- Defensive stance
	if PlayerClass == "WARRIOR" and GetShapeshiftForm(true) == 2 then
		return true
	end

	-- Righteous Fury
	if PlayerClass == "PALADIN" and SA:UnitHasBuff("player", buffRighteousFury) then
		return true
	end

	return false
end

local lastForm = nil
function mod:UPDATE_SHAPESHIFT_FORM()
	local currentForm = GetShapeshiftForm(true)
	if currentForm == lastForm then return end
	lastForm = currentForm
	
	-- Check for defensive stance and bear form
	if self.db.profile.Recheck and self:IsTanking() then
		state = false
	end
	self:ShouldCheck()
end

function mod:PLAYER_AURAS_CHANGED()
	self:ShouldCheck()
end

function mod:SpecialEvents_PlayerBuffGained(buff)
	if self.db.profile.Recheck and buff == buffRighteousFury then
		state = false
		self:ShouldCheck()
	elseif buff == buffSalvation or buff == buffGreaterSalvation then
		self:ShouldCheck()
	end
end

function mod:ShouldCheck()
	if not UnitInRaid("player") and not UnitInParty("player") then
		return false
	end

	if self.db.profile.WhileTankOnly and not self:IsTanking() then
		return false
	end

	self:Check()
end

function mod:Check()
	local hasBuff = self:HasSalvation()

	if (hasBuff and not state) then
		state = true

		local action = self.db.profile.Action
		if action == "query" then
			StaticPopup_Show("BosKiller")
		elseif action == "kill" then
			self:CancelBuff()
		elseif action == "keep" then
			-- Do nothing
		end
	end

	if (not hasBuff) then
		state = false
	end
end
